@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2009-12-13
+ *  Updated   : 2017-08-13
  *  Notes     :
  *
- * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2018 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,7 +69,7 @@ char dead_article[PATH_LEN];		/* ~/dead.article file */
 char dead_articles[PATH_LEN];		/* ~/dead.articles file */
 char default_organization[PATH_LEN];	/* Organization: */
 char default_signature[PATH_LEN];
-char domain_name[MAXHOSTNAMELEN];
+char domain_name[MAXHOSTNAMELEN + 1];
 char global_attributes_file[PATH_LEN];
 char global_config_file[PATH_LEN];
 char homedir[PATH_LEN];
@@ -99,7 +99,7 @@ char save_active_file[PATH_LEN];
 char spooldir[PATH_LEN];		/* directory where news is */
 char overviewfmt_file[PATH_LEN];	/* full path to overview.fmt */
 char subscriptions_file[PATH_LEN];	/* full path to subscriptions */
-char tin_progname[PATH_LEN];		/* program name */
+char *tin_progname;		/* program name */
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
 #ifdef HAVE_MH_MAIL_HANDLING
@@ -107,11 +107,11 @@ char userid[PATH_LEN];
 	char mailgroups_file[PATH_LEN];
 #endif /* HAVE_MH_MAIL_HANDLING */
 #ifndef NNTP_ONLY
-	char novfilename[PATH_LEN];		/* file name of a single nov index file */
+	char novfilename[NAME_LEN + 1];		/* file name of a single nov index file */
 	char novrootdir[PATH_LEN];		/* root directory of nov index files */
 #endif /* !NNTP_ONLY */
 
-t_function last_search;	/* for repeated search */
+t_function last_search = GLOBAL_SEARCH_REPEAT;	/* for repeated search */
 int hist_last[HIST_MAXNUM + 1];
 int hist_pos[HIST_MAXNUM + 1];
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
@@ -127,12 +127,15 @@ t_bool created_rcdir;			/* checks if first time tin is started */
 t_bool dangerous_signal_exit;		/* no get_respcode() in nntp_command when dangerous signal exit */
 t_bool disable_gnksa_domain_check;	/* disable checking TLD in From: etc. */
 t_bool disable_sender;			/* disable generation of Sender: header */
-t_bool filtered_articles;		/* locally killed / auto-selected articles */
 #ifdef NO_POSTING
 	t_bool force_no_post = TRUE;		/* force no posting mode */
 #else
 	t_bool force_no_post = FALSE;	/* don't force no posting mode */
 #endif /* NO_POSTING */
+#if defined(NNTP_ABLE) && defined(INET6)
+	t_bool force_ipv4 = FALSE;
+	t_bool force_ipv6 = FALSE;
+#endif /* NNTP_ABLE && INET6 */
 t_bool list_active;
 t_bool newsrc_active;
 t_bool no_write = FALSE;		/* do not write newsrc on quit (-X cmd-line flag) */
@@ -173,7 +176,7 @@ struct regex_cache
 		shar_regex,
 		slashes_regex, stars_regex, underscores_regex, strokes_regex
 #ifdef HAVE_COLOR
-		, quote_regex, quote_regex2, quote_regex3
+		, extquote_regex, quote_regex, quote_regex2, quote_regex3
 #endif /* HAVE_COLOR */
 	= {
 		NULL,
@@ -247,6 +250,7 @@ struct t_config tinrc = {
 	"",		/* quote_regex */
 	"",		/* quote_regex 2nd level */
 	"",		/* quote_regex >= 3rd level */
+	"",		/* extquote_regex */
 #endif /* HAVE_COLOR */
 	"",		/* slashes_regex */
 	"",		/* stars_regex */
@@ -269,7 +273,6 @@ struct t_config tinrc = {
 	0,		/* getart_limit */
 	2,		/* recent_time */
 	GOTO_NEXT_UNREAD_TAB,		/* goto_next_unread */
-	32,		/* groupname_max_length */
 	UUE_NO,	/* hide_uue */
 	KILL_UNREAD,		/* kill_level */
 	MIME_ENCODING_QP,		/* mail_mime_encoding */
@@ -280,6 +283,9 @@ struct t_config tinrc = {
 	SHOW_FROM_NAME,				/* show_author */
 	SORT_ARTICLES_BY_DATE_ASCEND,		/* sort_article_type */
 	SORT_THREADS_BY_SCORE_DESCEND,		/* sort_threads_type */
+#ifdef USE_HEAPSORT
+	0,				/* sort_function default qsort */
+#endif /* USE_HEAPSORT */
 	BOGUS_SHOW,		/* strip_bogus */
 	THREAD_BOTH,		/* thread_articles */
 	THREAD_PERC_DEFAULT,	/* thread_perc */
@@ -308,6 +314,7 @@ struct t_config tinrc = {
 	0,		/* col_quote (initialised later) */
 	0,		/* col_quote2 (initialised later) */
 	0,		/* col_quote3 (initialised later) */
+	0,		/* col_extquote (initialised later) */
 	0,		/* col_response (initialised later) */
 	0,		/* col_signature (initialised later) */
 	0,		/* col_urls (initialised later) */
@@ -327,6 +334,7 @@ struct t_config tinrc = {
 #ifdef HAVE_COLOR
 	FALSE,		/* use_color */
 #endif /* HAVE_COLOR */
+	FALSE,		/* abbreviate_groupname */
 	TRUE,		/* add_posted_to_filter */
 	TRUE,		/* advertising */
 	TRUE,		/* alternative_handling */
@@ -367,7 +375,6 @@ struct t_config tinrc = {
 	FALSE,		/* prompt_followupto */
 	QUOTE_COMPRESS|QUOTE_EMPTY,	/* quote_style */
 	TRUE,		/* show_description */
-	SHOW_INFO_LINES,		/* show_info */
 	TRUE,		/* show_only_unread_arts */
 	FALSE,		/* show_only_unread_groups */
 	TRUE,		/* show_signatures */
@@ -378,13 +385,24 @@ struct t_config tinrc = {
 #else
 	FALSE,		/* start_editor_offset */
 #endif /* M_UNIX */
+#ifndef USE_CURSES
 	TRUE,		/* strip_blanks */
+#endif /* !USE_CURSES */
 	FALSE,		/* strip_newsrc */
 	FALSE,		/* tex2iso_conv */
 	TRUE,		/* thread_catchup_on_exit */
 	TRUE,		/* unlink_article */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	FALSE,		/* utf8_graphics */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	TRUE,		/* verbatim_handling */
+#ifdef HAVE_COLOR
+	FALSE,		/* extquote_handling */
+#endif /* HAVE_COLOR */
 	"",		/* inews_prog */
+#ifdef USE_CANLOCK
+	1,			/* cancel_lock_algo, sha1 */
+#endif /* USE_CANLOCK */
 	INTERACTIVE_NONE,		/* interactive_mailer */
 	FALSE,		/* use_mouse */
 #ifdef HAVE_KEYPAD
@@ -401,9 +419,12 @@ struct t_config tinrc = {
 	FALSE,		/* use_slrnface */
 #endif /* XFACE_ABLE */
 	TRUE,		/* default_filter_select_global */
+	DEFAULT_SELECT_FORMAT,	/* select_format */
+	DEFAULT_GROUP_FORMAT,	/* group_format */
+	DEFAULT_THREAD_FORMAT,	/* thread_format */
 	DEFAULT_DATE_FORMAT,	/* date_format */
 #ifdef HAVE_UNICODE_NORMALIZATION
-	NORMALIZE_NFKC,		/* normalization form */
+	DEFAULT_NORMALIZE,		/* normalization form */
 #endif /* HAVE_UNICODE_NORMALIZATION */
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	FALSE,		/* render_bidi */
@@ -434,13 +455,17 @@ struct t_config tinrc = {
 #endif /* HAVE_ISPELL */
 	"",		/* attrib_quick_kill_scope */
 	"",		/* attrib_quick_select_scope */
+	"",		/* attrib_group_format */
+	"",		/* attrib_thread_format */
 	"",		/* attrib_date_format */
 	0,		/* attrib_trim_article_body */
 	0,		/* attrib_auto_cc_bcc */
-	SHOW_INFO_LINES,		/* attrib_show_info */
 	FILTER_SUBJ_CASE_SENSITIVE,		/* attrib_quick_kill_header */
 	FILTER_SUBJ_CASE_SENSITIVE,		/* attrib_quick_select_header */
 	MIME_ENCODING_QP,		/* attrib_mail_mime_encoding */
+#if defined(HAVE_ALARM) && defined(SIGALRM)
+	120,	/* nntp_read_timeout_secs */
+#endif /* HAVE_ALARM && SIGALRM */
 	MIME_ENCODING_8BIT,		/* attrib_post_mime_encoding */
 	POST_PROC_NO,			/* attrib_post_process_type */
 	SHOW_FROM_NAME,			/* attrib_show_author */
@@ -481,6 +506,9 @@ struct t_config tinrc = {
 	FALSE,		/* attrib_tex2iso_conv */
 	TRUE,		/* attrib_thread_catchup_on_exit */
 	TRUE,		/* attrib_verbatim_handling */
+#ifdef HAVE_COLOR
+	FALSE,		/* attrib_extquote_handling */
+#endif /* HAVE_COLOR */
 	FALSE,		/* attrib_x_comment_to */
 	TRUE,		/* attrib_wrap_on_next_unread */
 	FALSE,		/* attrib_ask_for_metamail */
@@ -520,6 +548,8 @@ struct t_capabilities nntp_caps = {
 	FALSE, /* AUTHINFO SASL */
 	FALSE, /* AUTHINFO available but not in current state */
 	SASL_NONE, /* SASL CRAM-MD5 DIGEST-MD5 PLAIN GSSAPI EXTERNAL OTP NTLM LOGIN */
+	FALSE, /* COMPRESS */
+	COMPRESS_NONE, /* COMPRESS_NONE, COMPRESS_DEFLATE */
 #if 0
 	FALSE, /* STREAMING: "MODE STREAM", "CHECK", "TAKETHIS" */
 	FALSE /* IHAVE */
@@ -528,7 +558,7 @@ struct t_capabilities nntp_caps = {
 	FALSE /* LISTGROUP doesn't select group */
 #else
 	TRUE
-#endif /*! BROKEN_LISTGROUP */
+#endif /* !BROKEN_LISTGROUP */
 };
 
 static char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
@@ -561,9 +591,10 @@ static const struct {
 	{ &tinrc.col_quote,       2 },
 	{ &tinrc.col_quote2,      3 },
 	{ &tinrc.col_quote3,      4 },
+	{ &tinrc.col_extquote,    5 },
 	{ &tinrc.col_response,    2 },
 	{ &tinrc.col_signature,   4 },
-	{ &tinrc.col_urls,       -1 },
+	{ &tinrc.col_urls,       DFT_FORE },
 	{ &tinrc.col_verbatim,    5 },
 	{ &tinrc.col_subject,     6 },
 	{ &tinrc.col_text,       DFT_FORE },
@@ -584,12 +615,13 @@ preinit_colors(
 
 void
 postinit_colors(
-	void)
+	int last_color)
 {
 	size_t n;
 
 	for (n = 0; n < ARRAY_SIZE(our_colors); n++) {
-		if (*(our_colors[n].colorp) == DFT_INIT) {
+		if (*(our_colors[n].colorp) == DFT_INIT
+		 || *(our_colors[n].colorp) >= last_color) {
 			switch (our_colors[n].color_dft) {
 				case DFT_FORE:
 					*(our_colors[n].colorp) = default_fcol;
@@ -619,9 +651,12 @@ init_selfinfo(
 {
 	FILE *fp;
 	char *ptr;
-	const char *cptr;
+	char tmp[PATH_LEN];
 	struct stat sb;
 	struct passwd *myentry;
+#if defined(DOMAIN_NAME) || defined(HAVE_GETHOSTBYNAME)
+	const char *cptr;
+#endif /* DOMAIN_NAME || HAVE_GETHOSTBYNAME */
 
 	domain_name[0] = '\0';
 
@@ -639,14 +674,14 @@ init_selfinfo(
 
 #ifdef DOMAIN_NAME
 	if ((cptr = get_domain_name()) != NULL)
-		strcpy(domain_name, cptr);
+		my_strncpy(domain_name, cptr, MAXHOSTNAMELEN);
 #endif /* DOMAIN_NAME */
 
 #ifdef HAVE_GETHOSTBYNAME
 	if (domain_name[0] == '\0') {
 		cptr = get_fqdn(get_host_name());
 		if (cptr != NULL)
-			strcpy(domain_name, cptr);
+			my_strncpy(domain_name, cptr, MAXHOSTNAMELEN);
 	}
 #endif /* HAVE_GETHOSTBYNAME */
 
@@ -657,10 +692,11 @@ init_selfinfo(
 
 	if ((myentry = getpwuid(getuid())) == NULL) {
 		error_message(2, _(txt_error_passwd_missing));
+		free(tin_progname);
 		giveup();
 	}
 
-	strcpy(userid, myentry->pw_name);
+	my_strncpy(userid, myentry->pw_name, sizeof(userid) - 1);
 
 	if (((ptr = getenv("TIN_HOMEDIR")) != NULL) && strlen(ptr)) {
 		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
@@ -673,11 +709,10 @@ init_selfinfo(
 
 	created_rcdir = FALSE;
 	dangerous_signal_exit = FALSE;
-	disable_gnksa_domain_check = FALSE;
+	disable_gnksa_domain_check = TRUE;
 	disable_sender = FALSE;	/* we set force_no_post=TRUE later on if we don't have a valid FQDN */
-	filtered_articles = FALSE;
 	iso2asc_supported = atoi(get_val("ISO2ASC", DEFAULT_ISO2ASC));
-	if (iso2asc_supported > NUM_ISO_TABLES || iso2asc_supported < 0) /* TODO: issue a warning here? */
+	if (iso2asc_supported >= NUM_ISO_TABLES || iso2asc_supported < 0) /* TODO: issue a warning here? */
 		iso2asc_supported = -1;
 	list_active = FALSE;
 	newsrc_active = FALSE;
@@ -801,7 +836,7 @@ init_selfinfo(
 		strncat(ptr, get_val("MM_CHARSET", MM_CHARSET), space);
 		if ((space -= strlen(ptr)) > 0) {
 			strncat(ptr, "\n", space);
-			match_list(ptr, "mm_network_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset);
+			match_list(ptr, "mm_network_charset=", txt_mime_charsets, &tinrc.mm_network_charset);
 		}
 		free(ptr);
 	}
@@ -818,8 +853,8 @@ init_selfinfo(
 
 	joinpath(rcdir, sizeof(rcdir), homedir, RCDIR);
 	if (stat(rcdir, &sb) == -1) {
+		my_mkdir(rcdir, (mode_t) (S_IRWXU)); /* TODO: bail out? give error message? no_write = TRUE? */
 		created_rcdir = TRUE;
-		my_mkdir(rcdir, (mode_t) (S_IRWXU));
 	}
 	strcpy(tinrc.mailer_format, MAILER_FORMAT);
 	my_strncpy(mailer, get_val(ENV_VAR_MAILER, DEFAULT_MAILER), sizeof(mailer) - 1);
@@ -867,18 +902,21 @@ init_selfinfo(
 	joinpath(postponed_articles_file, sizeof(postponed_articles_file), rcdir, POSTPONED_FILE);
 	joinpath(save_active_file, sizeof(save_active_file), rcdir, ACTIVE_SAVE_FILE);
 
-	snprintf(lock_file, sizeof(lock_file), INDEX_LOCK, TMPDIR, userid);
+	snprintf(tmp, sizeof(tmp), INDEX_LOCK, userid);
+	joinpath(lock_file, sizeof(lock_file), TMPDIR, tmp);
 
 #ifdef NNTP_ABLE
 	nntp_tcp_port = (unsigned short) atoi(get_val("NNTPPORT", NNTP_TCP_PORT));
 #endif /* NNTP_ABLE */
 
-	if (stat(posted_info_file, &sb) == -1) {
-		if ((fp = fopen(posted_info_file, "w")) != NULL) {
-			fprintf(fp, _(txt_posted_info_file));
-			fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
-			fclose(fp);
+	if ((fp = fopen(posted_info_file, "a")) != NULL) {
+		if (!fstat(fileno(fp), &sb)) {
+			if (sb.st_size == 0) {
+				fprintf(fp, "%s", _(txt_posted_info_file));
+				fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
+			}
 		}
+		fclose(fp);
 	}
 
 	init_postinfo();
@@ -956,12 +994,12 @@ read_site_config(
 		if (match_string(buf, "mm_charset=", tinrc.mm_charset, sizeof(tinrc.mm_charset)))
 			continue;
 #else
-		if (match_list(buf, "mm_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset))
+		if (match_list(buf, "mm_charset=", txt_mime_charsets, &tinrc.mm_network_charset))
 			continue;
 #endif /* !CHARSET_CONVERSION */
-		if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.post_mime_encoding))
+		if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, &tinrc.post_mime_encoding))
 			continue;
-		if (match_list(buf, "mail_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.mail_mime_encoding))
+		if (match_list(buf, "mail_mime_encoding=", txt_mime_encodings, &tinrc.mail_mime_encoding))
 			continue;
 		if (match_boolean(buf, "disable_gnksa_domain_check=", &disable_gnksa_domain_check))
 			continue;
@@ -1015,6 +1053,9 @@ postinit_regexp(
 	compile_regex(tinrc.strip_was_regex, &strip_was_regex, 0);
 
 #ifdef HAVE_COLOR
+	if (!strlen(tinrc.extquote_regex))
+		STRCPY(tinrc.extquote_regex, DEFAULT_EXTQUOTE_REGEX);
+	compile_regex(tinrc.extquote_regex, &extquote_regex, PCRE_CASELESS);
 	if (!strlen(tinrc.quote_regex))
 		STRCPY(tinrc.quote_regex, DEFAULT_QUOTE_REGEX);
 	compile_regex(tinrc.quote_regex, &quote_regex, PCRE_CASELESS);
@@ -1034,7 +1075,7 @@ postinit_regexp(
 	compile_regex(tinrc.stars_regex, &stars_regex, PCRE_CASELESS);
 	if (!strlen(tinrc.strokes_regex))
 		STRCPY(tinrc.strokes_regex, DEFAULT_STROKES_REGEX);
-	compile_regex(tinrc.strokes_regex,&strokes_regex, PCRE_CASELESS);
+	compile_regex(tinrc.strokes_regex, &strokes_regex, PCRE_CASELESS);
 	if (!strlen(tinrc.underscores_regex))
 		STRCPY(tinrc.underscores_regex, DEFAULT_UNDERSCORES_REGEX);
 	compile_regex(tinrc.underscores_regex, &underscores_regex, PCRE_CASELESS);
@@ -1066,7 +1107,7 @@ utf8_pcre(
 
 #	if (defined(PCRE_MAJOR) && PCRE_MAJOR >= 4)
 	(void) pcre_config(PCRE_CONFIG_UTF8, &i);
-#	endif /* PCRE_MAJOR && PCRE_MAJOR >= $*/
+#	endif /* PCRE_MAJOR && PCRE_MAJOR >= 4 */
 
 	return (i ? TRUE : FALSE);
 }
